@@ -25,20 +25,28 @@ False
 
 import requests
 import mimetypes
-from urllib import quote, urlencode
+import uuid
+
+try:
+    from urllib import quote, urlencode  # Python 2.X
+except ImportError:
+    from urllib.parse import quote, urlencode  # Python 3+
+
 from types import FunctionType
 from inspect import getsource
 from textwrap import dedent
 import re
 import socket
 import sys
-import hashlib
-import random
-from urlparse import urlparse
 
-from cStringIO import StringIO
 import json
 
+try:
+    basestring
+except NameError:
+    basestring = str
+
+
 __all__ = ['PreconditionFailed', 'ResourceNotFound', 'ResourceConflict',
            'ServerError', 'Server', 'Database', 'Document', 'ViewResults',
            'Row']
@@ -316,10 +324,15 @@ class Database(object):
         return iter([item.id for item in self.view('_all_docs')])
 
     def __len__(self):
+        print('len')
         """Return the number of documents in the database."""
         data = self.resource.get()
         return data['doc_count']
 
+    def __bool__(self):
+        # db is always true (for logic reason in document.create, for example)
+        return True
+
     def __nonzero__(self):
         """Return whether the database is available."""
         try:
@@ -851,7 +864,10 @@ class Document(dict):
             raise AttributeError(u'Field %s is undefined in this document' % name)
 
     def __setattr__(self, name, value):
-        if name == '_db':
+        ## if name == '_db':
+
+        # CouchDB is not allowing such field name. We can use for internal properties (@cached_property can also work)
+        if name.startswith('_') and name != '_id' and name != '_rev':
             super(Document, self).__setattr__(name, value)
         else:
             self[name] = value
@@ -876,7 +892,7 @@ class Document(dict):
         else:
             raise Exception('Can\'t delete document - target database is undefined')
 
-    def create(self, id, suffix_length = 12, max_retry = 100, db = None, force_random_suffix=False):
+    def create(self, id, suffix_length=12, max_retry=100, db=None, force_random_suffix=False):
         """Conflict-safe document creator.
         Document id will be '<id>' or '<id><random suffix>' if '<id>' already exists.
 
@@ -890,7 +906,7 @@ class Document(dict):
         if db:
             rand = ''
             if force_random_suffix:
-                rand = '_' + hashlib.sha1(str(random.random())).hexdigest()[:suffix_length]
+                rand = '_' + uuid.uuid4().hex[:suffix_length]
 
             doc_id = id_string % (id, rand)
 
@@ -906,11 +922,11 @@ class Document(dict):
                 if max_retry < 0:
                     raise Exception("Retry-limit reached during document generation")
 
-                rand = hashlib.sha1(str(random.random())).hexdigest()[:suffix_length]
+                rand = uuid.uuid4().hex[:suffix_length]
                 doc_id = id_string % (id, '_%s' % rand)
 
         else:
-            raise Exception('Can\'t create document - target database is undefined')
+            raise Exception("Can't create document - target database is undefined")
 
 
     def get_attachment(self, name):
@@ -932,8 +948,14 @@ class Document(dict):
 
         if rev != self.rev:
             doc = db[self.id]
+            delete = []
             for key in self.keys():
+                if not key in doc:
+                    delete.append(key)
+
+            for key in delete:
                 del(self[key])
+
             self.update(doc)
             return True
         else:
@@ -1235,7 +1257,7 @@ class Resource(object):
 
         body = None
         if content is not None:
-            if not isinstance(content, basestring):
+            if not isinstance(content, basestring) and not isinstance(content, bytes):
                 body = json.dumps(content).encode('utf-8')
             else:
                 body = content
